<?php
\JLoader::register('ApiCore', JPATH_PLUGINS . '/vmshipment/dellin_shipping/DellinClasses/ApiCore.php');
/**
 * Class DellinAPI
 * Класс предназначен для расчета стоимости доставки.
 */
class DellinApi extends ApiCore
{


    /**
     * Id для авторизации в API деловых линий
     * @var string
     */
    protected static $dellinSessionId = false;

	public static function GetCityKLADRCode($locationName, $state)
	{
		if (isset($locationName)) {
			$arName = explode(" ", trim($locationName));

			if ($arName > 3) {
				usort($arName, array('DellinApi', 'sort_func'));
				$searchString = implode(" ", array_slice($arName, 0, 2));
				$locationName = $searchString;
			}

			$dl_locations = self::SearchCity($locationName);
			if(empty($dl_locations)) return false;
		}
		if (count($dl_locations) > 0) {

			$dl_city = self::SelectCityByRegion($dl_locations,$locationName, $state);

			$kladr_code = (string)$dl_city->code;

		} else {
			return $dl_locations->errors;
		}
		return $kladr_code;
	}


	/**
     * Рассчитываем кол-во грузовых мест.
     * @param array $arOrder
     * @param array $arConfig
     * @return int
     */
    protected static function GetNumbersOfCargoPlaces($arOrder, $arConfig)
    {
        $numbers_of_places = 1;

        switch ($arConfig['LOADING_GROUPING_OF_GOODS']['VALUE']) {
            // Если считаем весь заказ, как 1 грузоместо.
            case 'ONE_CARGO_SPACE':
                break;

            // Если группируем каждый вид товара, как отдельное грузоместо.
            case 'SEPARATED_CARGO_SPACE':
                $numbers_of_places = count($arOrder['ITEMS']);
                break;

            // Если каждая единица товара - отдельное грузоместо.
            case 'SINGLE_ITEM_SINGLE_SPACE':
                $numbers_of_places = 0;

                foreach ($arOrder['ITEMS'] as $item) {
                    $numbers_of_places += $item['QUANTITY'];
                }
                break;
        }
        return $numbers_of_places;
    }

    /**
     * Конвертируем граммы в килограммы.
     * @param float $bx_goods_weight_in_gram
     * @return bool|float|int
     */
    protected static function ConvertWeightFromGramToKilogram($bx_goods_weight_in_gram)
    {
        $weight_in_kg = CSaleMeasure::Convert((float)$bx_goods_weight_in_gram, "G", "KG");
        return $weight_in_kg > 0 ? $weight_in_kg : 0.01;
    }

    /**
     * Получаем массив с параметрами веса, основываясь на перегрузе
     * @param float $bx_compare_weight
     * @param null|float $bx_set_weight
     * @return array
     */
    public static function GetWeightArray($bx_compare_weight, $bx_set_weight = null) {
        $compare_weight = self::ConvertWeightFromGramToKilogram($bx_compare_weight);
        $total_weight = is_null($bx_set_weight) ? $compare_weight : self::ConvertWeightFromGramToKilogram($bx_set_weight);

        if (ApiCore::IsOversizedWeight($compare_weight)) {
            return array('sized' => $total_weight, 'oversized' => $total_weight);
        }

        return array('sized' => $total_weight);
    }

    /**
     * Определяем параметры "sizedWeight" и "oversizedWeight" по способу группировки груза
     * @param array $arOrder
     * @param array $arConfig
     * @return array
     */
    static function CalculateWeight($arOrder, $arConfig)
    {
        $arWeight = array();
        switch ($arConfig['LOADING_GROUPING_OF_GOODS']['VALUE']) {
            // Если считаем весь заказ, как 1 грузоместо.

            case 'ONE_CARGO_SPACE':
                $arWeight = self::GetWeightArray($arOrder['WEIGHT']);
                break;

            // Если группируем каждый вид товара, как отдельное грузоместо.
            case 'SEPARATED_CARGO_SPACE':
                $max_item_weight = 0;
                $sum_item_weight = 0;
                foreach ($arOrder['ITEMS'] as $item) {
                    $item_weight = $item['WEIGHT'] * $item['QUANTITY'];
                    $sum_item_weight += $item_weight;

                    if ($item_weight > $max_item_weight) {
                        $max_item_weight = $item_weight;
                    }
                }

                $arWeight = self::GetWeightArray($max_item_weight, $sum_item_weight);
                break;

            // Если каждая единица товара - отдельное грузоместо.
            case 'SINGLE_ITEM_SINGLE_SPACE':
                $max_item_weight = 0;
                $sum_item_weight = 0;

                foreach ($arOrder['ITEMS'] as $item) {
                    $sum_item_weight += $item['WEIGHT'] * $item['QUANTITY'];

                    if ($item['WEIGHT'] > $max_item_weight) {
                        $max_item_weight = $item['WEIGHT'];
                    }
                }
                $arWeight = self::GetWeightArray($max_item_weight, $sum_item_weight);
                break;
        }

        if(empty($arWeight)) $arWeight['sized'] = 0.1;
        return $arWeight;
    }

    static function convertToM($value,$units){
		switch($units){
			case 'CM':
				return $value/100;
			break;
			case 'MM':
				return $value/1000;
			break;
			case 'M' :
				return $value;
			break;
		}
    }
	static function convertToKg($value,$units){
		switch($units){
			case 'G':
				return $value/1000;
				break;
			case 'T':
				return $value*1000;
				break;
			case 'KG' :
				return $value;
				break;
		}
	}
	static function productOversizeValidation($arDimensions,$stricts){
    	//ВАЖНО: ДЛИНА В STRICTS ДОЛЖНА БЫТЬ ЗАПОЛНЕНА САМЫМ БОЛЬШИМ ЗНАЧЕНИЕМ СТОРОНЫ (13.6м для фуры, к примеру).
    	//смотрим на каждый товар и проверяем, можно ли уложить его в фуру как есть и с поворотом. Если нет, вообще дальше не считаем
		$length = $arDimensions['length'];
		$width = $arDimensions['width'];
		$height = $arDimensions['height'];
		$totalWeight = $arDimensions['totalWeight'];
		$totalVolume = $arDimensions['totalVolume'];
		if($totalWeight >= $stricts['totalWeight']) return false;
		if($totalVolume >= $stricts['totalVolume']) return false;

		//если товар торчит из фуры/грузоместа по самой длинной стороне ('length'), то даже с поворотом груз не уложить
		if($length > $stricts['length'] || $width > $stricts['length'] || $height > $stricts['length']){
			return false;
		}
		$isOversized = true;
		//Если все стороны меньше минимального значения одной стороны, то товар габаритный.
		// Если есть сторона больше минимального значения (но меньше stricts['length']),
		// то 2 остальные не могут быть больше минимального значения, поэтому вертим товар во всех трех плоскостях.
		// Если находим такое положение, где товар не выпирает из габаритов, значит товар габаритный.
		if(
			($length <= $stricts['width_height'] && $length <= $stricts['width_height'] && $height<= $stricts['width_height'])||
			($length > $stricts['width_height'] && $width <= $stricts['width_height'] && $height <= $stricts['width_height']) ||
			($width > $stricts['width_height'] && $length <= $stricts['width_height'] && $height <= $stricts['width_height']) ||
			($height > $stricts['width_height'] && $length <= $stricts['width_height'] && $width <= $stricts['width_height'])
		){
			$isOversized = false;
		}
		if($isOversized) return false;

		//По итогу всех проверок, если общий вес/объем товаров вписывается в фуру/грузоместо
		// и товар можно положить так, чтобы он не выпирал из фуры/грузоместа,
		// значит он габаритный
		return true;
	}
	static function convertProductDimensions(&$products){
    	foreach($products as &$product){
		    $product->product_length = self::convertToM($product->product_length,$product->product_lwh_uom);
		    $product->product_width = self::convertToM($product->product_width,$product->product_lwh_uom);
		    $product->product_height = self::convertToM($product->product_height,$product->product_lwh_uom);
		    $product->product_lwh_uom = 'M';
		    $product->product_weight = self::convertToKG($product->product_weight,$product->product_weight_uom);
		    $product->product_weight_uom = 'KG';

	    }

	}
    static function getCargoParams($cart,$arConfig){
    	$products = $cart->products;

    	self::convertProductDimensions($products);

    	$maxProductLength = 0;
    	$maxProductHeight = 0;
    	$maxProductWidth = 0;
	    $weight = 0;
    	$totalWeight = 0;
    	$totalVolume = 0;
	    $oversizedVolume = 0;
	    $oversizedWeight = 0;
	    $freightName='';

    	foreach($products as $index=>$product){
    		if($product->product_length > $maxProductLength) $maxProductLength = $product->product_length;
    		if($product->product_height > $maxProductHeight) $maxProductHeight = $product->product_height;
    		if($product->product_width > $maxProductWidth) $maxProductWidth = $product->product_width;
    		$totalWeight += $product->product_weight*$product->amount;
    		$totalVolume += $product->product_length * $product->product_height * $product->product_width * $product->amount;
		    $freightName .= (($index != 0) ? ',' : '') . $product->product_name;
	    }
	    $arDimensions = [
	    	'length'=>$maxProductLength,
		    'width'=>$maxProductWidth,
		    'height'=>$maxProductHeight,
		    'totalWeight'=>$totalWeight,
		    'totalVolume'=>$totalVolume
	    ];
	    $globalStricts = [
		    'length'=>13.6,
		    'width_height'=>2.4,
		    'totalVolume' => 80,
		    'totalWeight' => 20000
	    ];
	    $placeStricts = [
		    'length' => 3,
		    'width_height'=>3,
		    'totalVolume'=>27,
		    'totalWeight' => 100
	    ];
	    $smallStricts = [
		    'length' => 0.54,
		    'width_height'=>0.39,
		    'totalVolume'=>0.1,
		    'totalWeight' => 10
	    ];

		if(self::productOversizeValidation($arDimensions,$globalStricts)){//проверяем габариты товаров, нет ли сликом огромных или тяжелых для всей фуры
			switch ($arConfig['loading_grouping_of_goods']) {
				//Если весь товар в одно грузоместо
				case 'ONE_CARGO_SPACE':
					$quantity  = 1;
					if(!self::productOversizeValidation($arDimensions, $placeStricts)){
						$oversizedVolume = $totalVolume;
						$oversizedWeight = $totalWeight;
					}
					$weight = $totalWeight;
					break;
				// Если группируем каждый вид товара, как отдельное грузоместо.
				case 'SEPARATED_CARGO_SPACE':
					$quantity = count($products);
					$maxPlaceWeight = 0;
					foreach($products as $product){
						$arDimensions = [
							'length' => $product->product_length,
							'width'=>$product->product_width,
							'height'=>$product->product_height,
							'totalWeight'=>$product->product_weight * $product->amount,
							'totalVolume'=>$product->product_length*$product->product_width*$product->product_height*$product->amount
						];
						if($arDimensions['totalWeight'] > $maxPlaceWeight) $maxPlaceWeight = $arDimensions['totalWeight'];
						if(!self::productOversizeValidation($arDimensions,$placeStricts)){
							$oversizedWeight += $arDimensions['totalWeight'];
							$oversizedVolume += $arDimensions['totalVolume'];
						}
					}
					$weight = $maxPlaceWeight;
					break;
				// Если каждая единица товара - отдельное грузоместо.
				case 'SINGLE_ITEM_SINGLE_SPACE':
					$quantity = 0;
					$maxProductWeight = 0;
					foreach ($products as $product){
						$arDimensions = [
							'length' => $product->product_length,
							'width'=>$product->product_width,
							'height'=>$product->product_height,
							'totalWeight'=>$product->product_weight,
							'totalVolume'=>$product->product_length*$product->product_width*$product->product_height
						];
						if($product->product_weight > $maxProductWeight) $maxProductWeight = $product->product_weight;
						if(!self::productOversizeValidation($arDimensions,$placeStricts)){
							$oversizedWeight += $arDimensions['totalWeight']*$product->amount;
							$oversizedVolume += $arDimensions['totalVolume'];
						}
						$quantity += $product->amount;
					}
					$weight = $maxProductWeight;

					break;
			}
			$small_available_packages = [
				'packing_for_goods_box',
				'packing_for_goods_additional',
				'packing_for_goods_bubble',
				'packing_for_goods_bag'

			];
			$packagesAvailable = true;
			foreach ($arConfig as $id=>$param){
				if(strpos($id,'packing_for_goods_') && !in_array($id,$small_available_packages)){
					$packagesAvailable = false;
				}
			}
			if($arConfig['loading_grouping_of_goods'] !== 'ONE_CARGO_SPACE' || $arConfig['is_small_goods_price'] != 1 || !self::productOversizeValidation($arDimensions,$smallStricts) || !$packagesAvailable || $arConfig['is_goods_loading'] != 1 || $arConfig['is_goods_unloading'] != 1 ){
				$deliveryType = 'auto';
			}else{
				$deliveryType = 'small';
			}
			$maxSide = $maxProductLength;
			if($maxProductWidth > $maxProductLength){
				$maxSide = $maxProductWidth;
				$maxProductWidth = $maxProductLength;
				$maxProductLength = $maxSide;
			}
			if($maxProductHeight > $maxProductLength){
				$maxSide = $maxProductHeight;
				$maxProductHeight = $maxProductLength;
				$maxProductLength = $maxSide;
			}
			$result = [
				'quantity'=>$quantity,
				'length'=>$maxProductLength?round($maxProductLength,2):0.01,
				'width'=>$maxProductWidth?round($maxProductWidth,2):0.01,
				'height'=>$maxProductHeight?round($maxProductHeight,2):0.01,
				'weight'=>$weight,
				'totalVolume'=>$totalVolume,
				'totalWeight'=>$totalWeight,
				'oversizedWeight'=>$oversizedWeight,
				'oversizedVolume'=>$oversizedVolume,
				'freightName' => $freightName,
				'deliveryType'=>$deliveryType
			];
		}else{
			return false;
		}

		return $result;
    }
    /**
     * Определяем параметры "sizedVolume" и "oversizedVolume" по способу группировки груза
     * @param array $arOrder
     * @param array $arConfig
     * @return array
     */
//    static function CalculateVolume($cart, $arConfig) {
//        $arVolume = array();
//
//        $totalVolume = 0;
//        $totalOversizedVolume = 0;
//	    $item_max_size = array('length' => 0, 'width' => 0, 'height' => 0);
//        foreach ($cart->products as $item) {
//            $item_length = (int) $item->product_length?self::convertToM($item->product_length,$item->product_lwh_uom):0;
//            $item_width = (int) $item->product_width?self::convertToM($item->product_width,$item->product_lwh_uom):0;
//            $item_height = (int) $item->product_height?self::convertToM($item->product_height,$item->product_lwh_uom):0;
//            $totalVolume += $item->amount * $item_length * $item_width * $item_height;
//	        if ($item_length > $item_max_size['length']) {
//		        $item_max_size['length'] = $item_length;
//	        }
//
//	        if ($item_width > $item_max_size['width']) {
//		        $item_max_size['width'] = $item_width;
//	        }
//
//	        if ($item_height > $item_max_size['height']) {
//		        $item_max_size['height'] = $item_height;
//	        }
//        }
//        switch ($arConfig['loading_grouping_of_goods']) {
//            // Если считаем весь заказ, как 1 грузоместо.
//            case 'ONE_CARGO_SPACE':
//                $arVolume = ApiCore::GetVolumeArrayBySize($item_max_size, $totalVolume);
//                break;
//
//            // Если группируем каждый вид товара, как отдельное грузоместо.
//            case 'SEPARATED_CARGO_SPACE':
//                foreach ($arOrder['ITEMS'] as $item) {
//                    $bx_item_length = $item['DIMENSIONS']['LENGTH'];
//                    $bx_item_width = $item['DIMENSIONS']['WIDTH'];
//                    $bx_item_height = $item['DIMENSIONS']['HEIGHT'];
//
//                    $bx_item_volume = $item['QUANTITY'] * $bx_item_length * $bx_item_width * $bx_item_height;
//
//                    if (ApiCore::IsOversizedVolume($bx_item_volume)) {
//                        $totalOversizedVolume += $bx_item_volume;
//                    }
//                }
//
//                $arVolume = ApiCore::GetVolumeArrayByVolume(
//                    $totalOversizedVolume,
//                    $totalVolume,
//                    ApiCore::ConvertVolumeFromCubicMillimeterToCubicMeter($totalOversizedVolume)
//                );
//
//                break;
//
//            // Если каждая единица товара - отдельное грузоместо.
//            case 'SINGLE_ITEM_SINGLE_SPACE':
//                foreach ($arOrder['ITEMS'] as $item) {
//                    $bx_item_length = $item['DIMENSIONS']['LENGTH'];
//                    $bx_item_width = $item['DIMENSIONS']['WIDTH'];
//                    $bx_item_height = $item['DIMENSIONS']['HEIGHT'];
//
//                    $bx_item_volume = $bx_item_length * $bx_item_width * $bx_item_height;
//
//                    if (ApiCore::IsOversizedVolume($bx_item_volume)) {
//                        $totalOversizedVolume += $bx_item_volume;
//                    }
//                }
//                $arVolume = ApiCore::GetVolumeArrayByVolume(
//                    $totalOversizedVolume,
//                    $totalVolume,
//                    ApiCore::ConvertVolumeFromCubicMillimeterToCubicMeter($totalOversizedVolume)
//                );
//
//                break;
//        }
//        if(empty($arVolume) || $arVolume['sized']<=0.001){
//            $arVolume['sized'] = 0.001;
//        }
//        return $arVolume;
//    }

//    /**
//     * Получаем значения для поля "{arrival-derival}Services"
//     * @param array $bx_fields
//     * @return array
//     */
//    protected static function GetAdditionalServices($bx_fields)
//    {
//        $value_list = array();
//
//        foreach ($bx_fields as $item) {
//            if ($item == 'NULL') {
//                continue;
//            }
//
//            $value_list[] = $item;
//        }
//        return $value_list;
//    }
//
    /**
     * Получаем массив из id типов упаковки.
     * @param array $bx_fields
     * @return array
     */
    public static function GetSelectedPackingTypesId(/*$requestType,*/$fields)
    {
//        if($requestType == 'calculator'){
//            $packagesArray = ApiCore::$packing_types_id;
//        }elseif($requestType == 'request'){
            $packagesArray = self::$request_packages_id;
//        }

        $value_list = array();

        foreach ($fields as $key => $item) {
            if (!isset($item) || $item !== '1') {
                continue;
            }

            $value_list[] = $packagesArray[$key];
        }
        return $value_list;
    }
    /**
     * Проверяем код на валидность.
     * @param array $data
     * @param array $arOrder
     * @param array $arConfig
     * @return bool
     */
    public static function IsDataValid($data, $cart, $arConfig)
    {

//        echo '<pre>';
//        var_dump($data);
//        echo '</pre>';
//        die();


        $errors = array();
        $status = 'ok';


        if((isset($cart->cost_shipment) && ($cart->cost_shpiment  <= 0)) ||
           (isset($cart->cartPrices['withTax']) && ($cart->cartPrices['withTax'] <= 0)) ){
            $errors['zero_order_price'] = vmText::_ ('VMSHIPMENT_ZERO_ORDER_PRICE');
        }
        if(  $data["appkey"] == ''){
            $errors['empty_app_key'] = vmText::_ ('VMSHIPMENT_NO_APP_KEY');
        }

        if(($data['delivery']['derival']['variant'] == 'terminal' && $data['delivery']['derival']['terminalID']=="") ||
           ($data['delivery']['derival']['variant'] == 'address' && (!isset($data['delivery']['derival']['address']['search'])
           ))){
	        $errors['empty_derival_point'] = vmText::_ ('VMSHIPMENT_EMPTY_DERIVAL_POINT');
        }

	    if(($data['delivery']['derival']['variant'] == 'terminal' && $data['delivery']['derival']['terminalID']=="") ||
            ($data['delivery']['derival']['variant'] == 'address' && (!isset($data['delivery']['derival']['address']['search'])))){
            $errors['empty_arrival_point'] = vmText::_ ('VMSHIPMENT_EMPTY_ARRIVAL_POINT');
        }

        if($data['delivery']['arrival']['variant'] == "terminal" || $data['delivery']['derival']['variant'] == "terminal")   {
	        if(($data['delivery']['arrival']['variant'] == 'terminal' && $arConfig['kladr_code_delivery_from'] == $data['delivery']['arrival']['city']) ||
		        ($data['delivery']['arrival']['variant'] == 'address' && (isset($data['delivery']['arrival']['address']['street']) && $arConfig['kladr_code_delivery_from'] == $data['delivery']['arrival']['address']['street']))
	        ){
		        $errors['same_city_terminal_delivery'] = vmText::_ ('VMSHIPMENT_SAME_CITY_TERMINAL_DELIVERY');
	        }
        }

        if(!empty($errors)){
            $status = 'error';
//            echo '<pre>';
//            var_dump($errors);
//            echo '</pre>';
//            die();
        }



	    return array(
		    'status'=> $status,
		    'errors' => $errors
	    );
    }
//
//    /**
//     * Формируем строку ошибки.
//     * @param $errors
//     * @return string
//     */
//    protected static function GetResponseErrors($errors)
//    {
//		if(isset($errors->messages) && is_array($errors->messages)){
//			 $errors_srt = implode('; ', $errors->messages);
//		}else{
//
//		}
//        return $errors_srt;
//    }
//
    /**
     * Расчет конечной стоимости.
     * @param object $response
     * @param array $arConfig
     * @return bool|float
     */
    protected static function CalculatePrice($response, $arConfig)
    {
        $total_price = 0;
        $is_small_goods_price = ($arConfig['is_small_goods_price'] == '1');
        if ($response->data->price > 0) {
            $total_price = $response->data->price;
        }
        $additional_price = $total_price - $response->data->availableDeliveryTypes->auto;
        if ($is_small_goods_price && isset($response->data->availableDeliveryTypes->small)) {
            $total_price = $response->data->availableDeliveryTypes->small + $additional_price;
        }

        /**/
        return $total_price;
    }
    /**
     * Расчет стоимости доставки.
     * @param array $cart
     * @param array $arConfig
     * @param bool $caching
     * @return array|bool
     */
    public static function Calculate($cart, $arConfig, $type = 'calc', $resetCache=true)
    {


        $data = self::GetDeliveryData($cart, $arConfig, $type);


        if(!isset($data['error'])){
            $validation = self::IsDataValid($data, $cart, $arConfig);

            if ($validation['status'] == 'ok') {
                jimport('joomla.cache.cache');
                $cache_id = 'dellin_shipping_calculate';
                $conf =\JFactory::getConfig();
                $options = array(
                    'defaultgroup' => 'plg_dellin_shipping',
                    'cachebase' => $conf->get('cache_path'),
                    'lifetime' => 10*60,
                    'language' => $conf->get('language'),
                    'storage' => 'file' );
                $cache = new \JCache($options);
                $cache->setCaching($conf->get('caching'));
                if($resetCache){


                    $response = DellinApi::sendApiRequest('calculator',$data);

                    if($response == null){
                        echo "<pre>";
                        var_dump($response);
                        echo "</pre>";
                        die();
                    }
                    if(is_array($response) && $response['errors'] == 'Unauthorized'){
                        $result['status'] = "error";
                        $result['body'] = $response->errors->message;
                        return $result;
                    }
                    if (isset($response->errors) && $response->errors !== null) {
                        if (isset($response->errors->message)) {
                            $result['status'] = "api_unavailable";
                            $result['body'] = $response->errors->message;
                            return $result;
                        }else{
                            $result['body'] = "";
                            foreach($response->errors as $index => $error){
                                if(isset($error->fields)) $errorFields = implode(' , ',$error->fields);
                                $result['status'] = "error";
                                $result['body'] .= "[".$error->code."]".$error->detail." Поля: ".$errorFields."| ";
                            }
                        }
                    }else{
                        unset($_SESSION['current_terminals']);
                        $_SESSION['current_terminals'] = $response->data->arrival->terminals;

                        if ($price = self::CalculatePrice($response, $arConfig))
                        {
                            $currentDate = new DateTime(date('Y-m-d'));
                            if($data['delivery']['arrival']['variant'] == 'terminal'){
                                $arrivalDate = new DateTime ($response->data->orderDates->arrivalToOspReceiver);
                            }else{
                                $arrivalDate = new DateTime ($response->data->orderDates->derivalFromOspReceiver);
                            }
                            $dateDiff = DellinApi::dateDifference($arrivalDate,$currentDate);
                            if($arConfig['is_goods_unloading'] == ''){
                                $_SESSION['shipping_days_terminal'] = $dateDiff;
                            } else {
                                $_SESSION['shipping_days_address'] = $dateDiff;
                            }

                            $result['status'] = 'OK';
                            $result['body']   = array($price, $dateDiff);
                        }
                        if(isset($response->data->availableDeliveryTypes->small) && $arConfig['is_small_goods_price']=="1" ){
                            $result['delivery_type'] = "small";
                        }else{
                            $result['delivery_type'] = "auto";
                        }
                        $cache->store(serialize($result),$cache_id,'plg_dellin_shipping');
                    }
                }else{

                    $result = unserialize($cache->get($cache_id));

                }

                return $result;
            }else{
                return [
                    'status' => 'error',
                    'errors' => $validation['errors']
                ];

            }
        }else{
            return [
                'status'=>'error',
                'errors'=>$data['errors']
            ];
        }
    }

    protected static function isCashOnDelivery($id){
        if (!class_exists('VirtueMartModelPaymentmethod'))
            require(JPATH_VM_ADMINISTRATOR.DS.'models'.DS.'paymentmethod.php');
        $paymentObj= new VirtueMartModelPaymentmethod;
        $payment = $paymentObj->getPayment($id);
        if($payment->slug == 'cash-on-delivery') return true;
        return false;
    }


    /**
     * Метод проверяющий наложный платёж.
     *
     * @param $dataInOrder
     * @return bool
     */
    protected static function isCacheOnDelivery($dataInOrder){
        return $dataInOrder == 'Cash on delivery';
    }



    protected function getAddressForCDI($cart, $type)
    {
        if($type == 'request'){

            return $cart->person['state'].', '.$cart->person['city'].', '.$cart->person['address'];

        } else {

            $city = isset($cart->ST['city'])?$cart->ST['city']:$cart->BT['city'];
            $state = isset($cart->ST['virtuemart_state_id'])?ShopFunctions::getStateByID($cart->ST['virtuemart_state_id']):
                ShopFunctions::getStateByID($cart->BT['virtuemart_state_id']);
            $address_1 = isset($cart->ST['address_1'])?$cart->ST['address_1']:$cart->BT['address_1'];
            $address_2 = isset($cart->ST['address_2'])?$cart->ST['address_2']:$cart->BT['address_2'];
            $address = $address_1.' '.$address_2;

            return $state.' '.$city.' '.$address;
        }
    }

    protected static function getCityAndState($cart, $type)
    {

        if($type == 'request'){

            $result = [
                'state' => $cart->person['state'],
                'city' => $cart->person['city']
            ];

            return $result;

        } else {
            $city = isset($cart->ST['city'])?$cart->ST['city']:$cart->BT['city'];
            $state = isset($cart->ST['virtuemart_state_id'])?ShopFunctions::getStateByID($cart->ST['virtuemart_state_id']):
                ShopFunctions::getStateByID($cart->BT['virtuemart_state_id']);
            $result = [
                'city' => $city,
                'state' => $state
            ];

            return $result;
        }

        return false;

    }


    protected static function getArrivalTerminal($cart)
    {

        $db = JFactory::getDBO ();
        $prefix = $db->getPrefix();
        $q = 'SELECT * FROM `'.$prefix.'virtuemart_shipment_plg_dellin_shipping` '
            . 'WHERE `virtuemart_order_id` = ' . $cart->order_id;
        $db->setQuery ($q);
        if (!($dellininfo = $db->loadObject())) {
            $msg=vmText::sprintf('VMSHIPMENT_WEIGHT_COUNTRIES_NO_ENTRY_FOUND', $cart->order_id);
            vmWarn ($msg);
            vmDebug($msg, $q . " " . $db->getErrorMsg ());
            return '';
        }

        return $dellininfo->order_terminal_id;
    }

    protected static function getTrackingId($cart)
    {

        $db = JFactory::getDBO ();
        $prefix = $db->getPrefix();
        $q = 'SELECT * FROM `'.$prefix.'virtuemart_shipment_plg_dellin_shipping` '
            . 'WHERE `virtuemart_order_id` = ' . $cart->order_id;
        $db->setQuery ($q);
        if (!($dellininfo = $db->loadObject())) {
            $msg=vmText::sprintf('VMSHIPMENT_WEIGHT_COUNTRIES_NO_ENTRY_FOUND', $cart->order_id);
            vmWarn ($msg);
            vmDebug($msg, $q . " " . $db->getErrorMsg ());
            return '';
        }
        return $dellininfo->order_terminal_id;
    }


    protected static function GetDeliveryData($cart, $arConfig, $type = 'calc'){


        $sessionID = DellinApi::getDellinSessionId($arConfig);

        $deliveryType = 'auto';
        $calcCity = isset($cart->ST['city']) || isset($cart->BT['city']) || isset($cart->person['city']);
        $calcCityCalc = (isset($cart->BT['city']))?$cart->BT['city']:$cart->ST['city'];
  //    $calcCityValue  = ($type == 'request')? $cart->person['city'] : $calcCityCalc;

        if($calcCity){

            $city = self::getCityAndState($cart, $type)['city'];
            $state = self::getCityAndState($cart, $type)['state'];

            $receiverKladr = DellinApi::GetCityKLADRCode($city, $state);
            if($arConfig['is_small_goods_price'] == '1'){
                $deliveryType = 'small';
            }
            $opfList = DellinApi::GetOpfList($arConfig['appkey'])['list'];
            if(isset($opfList)){
                foreach($opfList as $opf){
                    $arOpfList[$opf['uid']] = $opf;
                }
            }


            //коды ИП для разных стран
            $receiverPPCodes = array(
                "176" => "0xab91feea04f6d4ad48df42161b6c2e7a"
            );

//            echo '<pre>';
//           // var_dump($cart);
//            var_dump($receiverPPCodes[$cart->shipping_country]);
//            echo '</pre>';
//            die();


            $cargoParams = DellinApi::getCargoParams($cart,$arConfig);


            if(!$cargoParams) {
                $errors['cargo_global_oversize'] = vmText::_ ('VMSHIPMENT_CARGO_GLOBAL_OVERSIZE');
            }
            if($arConfig['delivery_delay']=="" || $arConfig['delivery_delay'] == 0){
                $deliveryDelay = 1;
            }else{
                $deliveryDelay = $arConfig['delivery_delay'];
            }
            $produceData = array(
                "appKey"                => $arConfig['appkey'],
                "produceDate"           => date("Y-m-d",mktime(0, 0, 0, date("m")  , date("d")+$deliveryDelay, date("Y"))), //ориентировочная минимальная дата (сегодня + дней из настроек)
                "sizedVolume"           => $cargoParams['totalVolume'],
                "sizedWeight"           => $cargoParams['totalWeight'],
                "derivalPoint"          => $arConfig['kladr_code_delivery_from'],
                "derivalDoor"           => ($arConfig['is_goods_loading'] == "1")?true:false,
                "deliveryType"          => "1",
                "derivalPeriodVisit"    => array(
                    "workStart" => $arConfig['work_start'],
                    "workEnd"   => $arConfig['work_end']
                ),
                "oversizedWeight" => $cargoParams['oversizedWeight'],
                "oversizedVolume" => $cargoParams['oversizedVolume'],
                "length"          => $cargoParams['length'],
                "width"           => $cargoParams['width'],
                "height"          => $cargoParams['height']
            );


            $produceDate = DellinApi::GetProduceDate($produceData, $arConfig);



            if(!empty($produceDate)){
                if(!isset($produceDate->errors)){
                    if($produceDate->nearestDate){//если дата не подошла
                        $nearestDate = $produceDate->nearestDate;
                    }else{
                        $nearestDate = date("Y-m-d",mktime(0, 0, 0, date("m")  , date("d")+$deliveryDelay, date("Y")));
                    }
                }else{
                    $errors['produce_date_error'] = vmText::_ ('VMSHIPMENT_PRODUCE_DATE_ERROR');
                    $errors['produce_date_errors'] = $produceDate->errors;
                }
                $cargoParams['insurance'] = [
                    "statedValue" =>($arConfig['is_insurance_goods_with_declared_price']=='1')?($cart->cartPrices['withTax']):0,
                    "payer"       => self::isCashOnDelivery($cart->virtuemart_paymentmethod_id)?"receiver":"sender",
                    "term"        => false
                ];
                $api_data =[
                    'appkey'=>$arConfig['appkey'],
                    'sessionID'=>$sessionID,
                    'delivery'=>[
                        "deliveryType"=>[
                            "type"=>$cargoParams['deliveryType']
                        ],
                        'arrival'=>[],
                        'derival'=>[
                            'produceDate'=>$nearestDate
                        ],
                        'packages'=>[],
                    ],
                    'members'=>[
                        "requester" => [
                            "role"  => "sender",
                        ],
                    ],
                    'cargo'=>$cargoParams,
                    'payment'=>[
                        'paymentCity'=>$arConfig['kladr_code_delivery_from'],
                        'type'=>'noncash'
                    ],
                    'productInfo'=>array(
                        'type'=> 4,
                        'productType'=> 5,
                        'info'=>array(
                            array(
                                'param'=> 'module version',
                                'value'=> 'beta'
                            )
                        )
                    )
                ];

                //если нам нужны данные для реквеста их надо дополнить
                //получаем данные для реквеста

                if($type == 'request'){

                    $api_data['delivery']['deliveryType']['payer'] = self::isCacheOnDelivery($cart->payment_method_name)?"receiver":"sender";
                    $api_data['delivery']['derival']['variant'] = ($arConfig['is_goods_loading'] =='yes')?'address':'terminal';
                    $api_data['delivery']['derival']['payer'] = self::isCacheOnDelivery($cart->payment_method_name)?"receiver":"sender";

                    $api_data['delivery']['arrival']['variant'] = ($arConfig['is_goods_unloading'] =='yes')?'address':'terminal';
                    $api_data['delivery']['arrival']['payer'] = self::isCacheOnDelivery($cart->payment_method_name)?"receiver":"sender";



                    $api_data['members'] = array(
                        'requester' => array(
                            'role'  => 'sender',
                            'email' => $arConfig['sender_contact_email']
                        ),
                        'sender'=> array(
                            'counteragent'=> array(
                                'form'=> $arConfig['sender_form'],
                                'name'=> $arConfig['sender_name'],
                                'inn'=> $arConfig['sender_inn'],
                                "juridicalAddress" => ($arOpfList[$arConfig['sender_form']]['juridical'] == 1)?array(
                                    'search' => $arConfig['sender_juridical_kladr_street']
//								"street"    => $arConfig['sender_juridical_kladr_street'],
//								"house"     => $arConfig['sender_juridical_house_num'],
                                ):array(),
                                "isAnonym" => false
                            ),
                            'contactPersons' => array(
                                array(
                                    "name" => $arConfig['sender_contact_name'],
                                    "save" => false
                                )
                            ),
                            "phoneNumbers" => array(
                                array(
                                    "number" => preg_replace("/[^0-9]/", '', $arConfig['sender_contact_phone'])
                                )
                            ),
                            'email' => $arConfig['sender_contact_email']
                        ),
                        'receiver'=> array(
                            'counteragent'=> array(
                                'form' =>  $receiverPPCodes[$cart->shipping_country], //физическое лицо. Если юр. лицо, то ниже counteragent изменится на массив параметров для юр. лица
                                'name' =>  ''.$cart->person['last_name'].' '.$cart->person['first_name'].''
                            ),
                            'contactPersons'=> array(
                                array(
                                    'name'=> $cart->person['last_name'].' '.$cart->person['first_name'],
                                    'save'=> false
                                ),
                            ),
                            'phoneNumbers'=> array(
                                array(
                                    "number" => $str = preg_replace("/[^0-9]/", '', $cart->person['phone'])
                                ),
                            ),
                            "email" => $cart->person['email']
                        ),
                    );

                    $api_data['payment']['primaryPayer'] = 'sender';

                    //Для
//                    if($cart->person_type == '1'){
//                        $api_data['members']['receiver']['counteragent'] = array(
//                            "name" => $cart->organisation_name,
//                            "inn"  => $cart->organisation_inn,
//                            "juridicalAddress" => array(
//                                "search"    => $cart->organisation_address ,
//                            ),
//                            "isAnonym" =>false
//                        );
//                        $api_data['members']['receiver']['dataForReceipt'] = array('send'=>false);
//                    }
                    //
                    if($cart->person_type == '0'){
                        $api_data['members']['receiver']['counteragent']['name'] = $cart->person['last_name'].' '.$cart->person['first_name'];
                        $api_data['members']['receiver']['counteragent']['phone'] =  preg_replace("/[^0-9]/", '', $cart->person['phone']);
                        $api_data['members']['receiver']['counteragent']['isAnonym'] = true;
                        $api_data['members']['receiver']['dataForReceipt'] = array('send'=>false);
                    }


                    if(self::isCacheOnDelivery($cart->payment_method_name)){
                        $api_data['payment']['type'] = 'cash';
                        $api_data['payment']['primaryPayer'] = 'receiver';
                        $api_data['payment']['cashOnDelivery'][0]['orderNumber'] = intval($cart->order_id);
                        $api_data['payment']['cashOnDelivery'][0]['orderDate'] = date('Y-m-d',strtotime($cart->created_date));
                        $api_data['payment']['cashOnDelivery'][0]['paymentType']  = 'cash';
                    }


                }


                if($arConfig['is_goods_loading'] == "1"){
                    $api_data['delivery']['derival']['variant'] = 'address';
                    if(!empty($arConfig['loading_address'])){
                        $api_data['delivery']['derival']['address']['search'] = $arConfig['loading_address'];
                    }else{
                        $api_data['delivery']['derival']['address']['street'] = $arConfig['kladr_code_delivery_from'];
                    }

                  //  $api_data['delivery']['derival']['address']['street'] = $arConfig['kladr_code_delivery_from'];
                    $api_data['delivery']['derival']['time'] = array(
                        'worktimeStart' => $arConfig['work_start'],
                        'worktimeEnd'   => $arConfig['work_end'],
                    );
                    if("NULL" !== $arConfig['work_break_start'] && "NULL" !== $arConfig['work_break_end']){
                        $api_data['delivery']['derival']['time']['breakStart'] = $arConfig['work_break_start'];
                        $api_data['delivery']['derival']['time']['breakEnd'] = $arConfig['work_break_end'];

                    }

                }else{
                    $api_data['delivery']['derival']['variant'] = 'terminal';
                    $api_data['delivery']['derival']['terminalID'] = $arConfig['terminal_id'];
                }

                if($arConfig['loading_transport_requirements']!== "NULL") $requirements[] = $arConfig['loading_transport_requirements'];
                if($arConfig['loading_additional_equipment'] !== "NULL") $requirements[] = $arConfig['loading_additional_equipment'];
                if(!empty($requirements)) $deliveryData["delivery"]["derival"]['requirements'] = $requirements;

                if($arConfig['is_goods_unloading'] == "1"){
                    $api_data['delivery']['arrival']['variant'] = 'address';
                    $api_data['delivery']['arrival']['address']['search'] = self::getAddressForCDI($cart, $type);
                    $api_data['delivery']['arrival']['time'] = [
                        'worktimeStart' => $arConfig['work_start'],
                        'worktimeEnd'   => $arConfig['work_end'],

                    ];

                    if("NULL" !== $arConfig['work_break_start'] && "NULL" !== $arConfig['work_break_end']){
                        $api_data['delivery']['arrival']['time']['breakStart'] = $arConfig['work_break_start'];
                        $api_data['delivery']['arrival']['time']['breakEnd'] = $arConfig['work_break_end'];
                    }

                }else{
                    $api_data['delivery']['arrival']['variant'] = 'terminal';
                  //  $api_data['delivery']['arrival']['city'] = $receiverKladr;
                    $api_data['delivery']['arrival']['terminalID'] = self::getArrivalTerminal($cart);
                }
                $arPackages = DellinAPI::GetSelectedPackingTypesId(
                    [
                        'box'=> $arConfig['packing_for_goods_box'],
                        'hard' => $arConfig['packing_for_goods_hard'],
                        'additional' => $arConfig['packing_for_goods_additional'],
                        'bubble' => $arConfig['packing_for_goods_bubble'],
                        'bag' => $arConfig['packing_for_goods_bag'],
                        'pallet' => $arConfig['packing_for_goods_pallet'],
                        'car_glass' =>$arConfig['packing_for_goods_car_glass'],
                        'car_parts' => $arConfig['packing_for_goods_car_parts'],
                        'complex_pallet' => $arConfig['packing_for_goods_complex_pallet'],
                        'complex_hard' => $arConfig['packing_for_goods_complex_hard'],
                    ]
                );
                foreach($arPackages as $packageUid){
                    $arPackage = ['uid'=>$packageUid,'payer'=>'sender'];
                    //если мешки или коробки, тоставим количество 1, т.к. поле обязательное, но мы не можем предсказать, сколько их нужно
                    if($packageUid == "0x82750921BC8128924D74F982DD961379"
                        || $packageUid=="0x947845D9BDC69EFA49630D8C080C4FBE"
                        || $packageUid=="0xad97901b0ecef0f211e889fcf4624fed"
                        || $packageUid=="0xad97901b0ecef0f211e889fcf4624fea"
                    ) $arPackage['count']=1;
                    $packages[] = $arPackage;
                }
                if(isset($packages)) $api_data["delivery"]["packages"] = $packages;
                if("NULL" !== $arConfig['counteragent']){
                    $api_data['members']['requester']['uid'] = $arConfig['counteragent'];
                }

                return $api_data;


            }else{
                $errors['api_not_available'] = vmText::_ ('VMSHIPMENT_DELLIN_SHIPPING_API_NOT_AVAILABLE');
            }

        }else{
            $errors['empty_arrival_point'] = vmText::_ ('VMSHIPMENT_EMPTY_ARRIVAL_POINT');
        }
        if(!empty($errors)) return ['error'=>true,'errors'=>$errors];
    }


	public static function GetProduceDate($data, $deliveryConfig, $baseDate=false){

		if(!$baseDate) $baseDate = $data['produceDate'];

		$produceDate = self::sendApiRequest('produceDate',$data,'json');


		if(!isset($produceDate)){
			return false;
		}
		if(isset($produceDate->errors)){
			return $produceDate->errors;
		}
		if ($produceDate->produce_available) {//проверяем, возможна ли отгрузка хоть на какой-нибудь терминал в городе в эту дату

			if($deliveryConfig['is_goods_loading'] == "") {
				//если да, проверяем расписание терминала
				if(!empty($deliveryConfig['terminal_id'])){
					$terminal = self::GetTerminals($deliveryConfig['api_key'],$deliveryConfig['kladr_code_delivery_from'], $deliveryConfig['terminal_id'])['terminal'];
					foreach ($terminal->worktables->worktable as $worktable) {
						$departmentname = trim($worktable->department);
						if ($departmentname ==  vmText::_ ('VMSHIPMENT_DEPT1') || $departmentname == vmText::_ ('VMSHIPMENT_DEPT2')) {
							$weekDay = strtolower(date('l',strtotime($data['produceDate'])));
							if ($worktable->{$weekDay} !== "-" ) {
								if(str_replace(" ","",$worktable->{$weekDay}) == vmText::_ ('VMSHIPMENT_DEPT2')){
									$startTime = strtotime(date('Y-m-d') . " " . "00:00");
									$endTime =  strtotime(date('Y-m-d') . " " . "23:59");
									$workHours = array($startTime,$endTime);
								}else{
									$workHours = explode("-", $worktable->{$weekDay});
									$startTime = strtotime(date('Y-m-d') . " " . $workHours[0]);
									$endTime = ($workHours[1] == '24:00')? strtotime(date('Y-m-d') . " 23:59"):strtotime(date('Y-m-d') . " " . $workHours[1]);
								}

								$shopStartTime = strtotime((($deliveryConfig['work_start']<10)?"0":"").$deliveryConfig['work_start'].":00");
								$shopEndTime = strtotime((($deliveryConfig['work_end']<10)?"0":"").$deliveryConfig['work_end'].":00");

								if ($shopEndTime <= $startTime || $shopStartTime >= $endTime) {
									$produceDate->changeTime = $workHours;
								}
								$produceDate->nearestDate = $data['produceDate'];
								break;
							}else{

								$oldProduceDate = new DateTime($data['produceDate']);
								$newProduceDate = $oldProduceDate->modify('+1 day');
								$datediff = self::dateDifference($newProduceDate->format('Y-m-d'),$baseDate);
								if($datediff <=2){
									$data['produceDate'] = $newProduceDate->format('Y-m-d');
									$produceDate = self::GetProduceDate($data,$deliveryConfig,$baseDate);
									$produceDate->DATE_CHANGED = true;
								}
							}
						}
					}
					if(!isset($produceDate->nearestDate)){
						$produceDate->produce_available = false;
						$produceDate->errors['error_terminal_department'] = vmText::_('VMSHIPMENT_ERROR_TERMINAL_DEPARTMENT');
					}
				}else{
					$produceDate->errors['error_terminal_id'] = vmText::_('VMSHIPMENT_ERROR_TERMINAL_ID');
				}
			}else{
				$produceDate->nearestDate = $data['produceDate'];
			}
		}else{
			$oldProduceDate = new DateTime($data['produceDate']);
			$newProduceDate = $oldProduceDate->modify('+1 day');
			$datediff = self::dateDifference($newProduceDate->format('Y-m-d'),$baseDate);
			if($datediff <=2){
				$data['produceDate'] = $newProduceDate->format('Y-m-d');
				$produceDate = self::GetProduceDate($data,$deliveryConfig,$baseDate);
				$produceDate->DATE_CHANGED = true;
			}
			else{
				echo "<pre>";
				var_dump(vmText::_('VMSHIPMENT_ERROR_NEAREST_DATE'));
				echo "</pre>";
				die();
			}
		}

		return $produceDate;
	}

	/**
	 * Разница между датами
	 * @param $date_1
	 * @param $date_2
	 * @param string $differenceFormat
	 * @return string
	 */
	public static function dateDifference($date_1 , $date_2 , $differenceFormat = '%a' )
	{
        if(!is_object($date_1) || !is_object($date_2)){
            $date_1 = date_create($date_1);
            $date_2 = date_create($date_2);
        }

		$interval = date_diff($date_1, $date_2);
		return $interval->format($differenceFormat);

	}

	public static function getCounterAgents($data){
		$arCounterAgents = [];
		$counterAgentsData = ApiCore::sendApiRequest('requestCounteragents',$data);
		if(!empty($counterAgentsData) && !isset($counterAgentsData->errors)) {
			$counterAgents = $counterAgentsData->counteragents;
			foreach ($counterAgents as $counterAgent) {
				if ($counterAgent->uid) {
					$arCounterAgents[$counterAgent->uid] = $counterAgent;
				}
			}
		}
		return $arCounterAgents;
	}

	public static function getOpf($appkey){
		$arCountriesList = [];
		$arOpfList = [];
		if(isset($appkey)){
			$opfList = ApiCore::GetOpfList($appkey)['list'];
			if(isset($opfList)){
				$countries = ApiCore::GetCountries($appkey);

				foreach($countries as $country){
					$arCountriesList[$country['countryUID']] = $country['country'];
				}
				asort($arCountriesList);

				foreach($opfList as $id => $opf){
					$arOpfList[$opf['countryUID']][$opf['uid']] = $opf['name'];
					asort( $arOpfList[$opf['countryUID']]);
				}

			}
			return array('opf'=>$arOpfList,'countries'=>$arCountriesList);
		}
	}




	public static function getRequestTerminals($data){
		$terminalsOb = ApiCore::sendApiRequest('requestTerminal',$data);
		$terminalsInfo = ApiCore::GetTerminals($data['appkey']);
		$arTerminalIdValues = Null;
		if(isset($terminalsInfo) && !empty($terminalsInfo)){
			foreach ($terminalsOb->terminals as $terminal) {
				if(isset($terminalsInfo[$terminal->city_code]['terminals'][$terminal->id]) && $terminalsInfo[$terminal->city_code]['terminals'][$terminal->id]->receiveCargo){ //если терминал может принять товар
					$arTerminalIdValues[$terminal->id] = $terminal->address;
				}
			}
		}
		return $arTerminalIdValues;
	}

    /**
     * @param $idOrder
     * @param bool $orderUpdate
     * @param null $itemId
     *
     * @return array|bool|mixed|string[]
     */
    static function CreateOrder($cart, $config, $orderUpdate = false, $itemId = null){


        if($orderUpdate !== true || $orderUpdate = false) {

            $calcResult = self::ReCalculateOrder($config, $cart);

            if ( $calcResult['status'] !== 'error' && $calcResult['STATUS'] !== 'API_UNAVAILABLE' ) {

                if ( $calcResult['PRICE_CHANGED'] ) {

                    $result = $calcResult;
                    try {
                        //Проверяем указан ли айдишник метаполя где содержится стоимость, если указана изменяем заказ.
                        if ( ! is_null( $itemId ) ) {
                            if ( wc_update_order_item_meta( $itemId, 'cost', $calcResult['body']['price'], $cart->totalPrice ) ) {
                                $body   = array( "STATUS" => "OK", 'orderUpdate' => true,
                                    'data'=> array('state'=>'processing'));
                                $result = $body;
                            }
                        }
                    } catch ( Exception $e ) {
                        // Если произошла ошибка, пробуем транзакцией откатить и возвращаем ошибку.
                        wc_transaction_query( 'rollback' );

                        return new WP_Error( 'checkout-error', $e->getMessage() );
                    }
                } else {

                    $data = self::GetDeliveryData( $cart, $config, 'request' );
                    if ( isset( $data->errors ) ) {
                        $errorBody = 'Ошибко';//todo убрать
                        if ( is_object( $data->errors ) ) {
                            $arCategories = get_object_vars( $data->errors );
                            foreach ( $arCategories as $catCode => $error ) {
                                $arErrors = get_object_vars( $errors );
                                foreach ( $arErrors as $code => $error ) {
                                    $errorBody .= '[' . $code . ']' . $error . '<br/>';
                                }
                            }
                        } else {
                            foreach ( $data->errors as $code => $error ) {
                                $errorBody .= '[' . $code . ']' . $error . '<br/>';
                            }
                        }
                        $result = array( 'STATUS' => 'ERROR', 'BODY' => $errorBody );
                    } else {

                        $result = ApiCore::sendApiRequest( 'request', $data );

                        if ( $result->data->state == 'success' ) {

                            $db = JFactory::getDBO ();
                            $prefix = $db->getPrefix();

                            $table = $prefix.'virtuemart_shipment_plg_dellin_shipping';

                            $q = 'UPDATE `'.$table.'` '
                                .'SET `tracking_id` = '.$result->data->requestID.' '
                                .'WHERE `virtuemart_order_id` = '. $cart->order_id;
                            $db->setQuery ($q);

                            if (!($methodInfo = $db->loadObject())) {
                                $msg=vmText::sprintf('VMSHIPMENT_WEIGHT_COUNTRIES_NO_ENTRY_FOUND', $cart->order_id);
                                vmWarn ($msg);
                                vmDebug($msg, $q . " " . $db->getErrorMsg ());
                                return $result;
                            }

//                            update_post_meta( $idOrder, '_dellin_track_id', '' . $result->data->requestID . '', 'None' );
                        }
                    }
                }
            } else {
                if ($calcResult['status'] == 'error'){
                    $errorBody = '';
                    if ( isset( $calcResult['body'] ) ) {
                        $errorBody .= $calcResult['body'];
                    }
                    $result = array( 'status' => 'error', 'body' => $errorBody );
                }

            }

        }


        return $result;

    }

    /**
     * Пересчет заказа перед созданием заявки
     */
    function ReCalculateOrder($config, $cart){

        $calcResult = self::Calculate($cart, $config, $type = 'request');


        if($calcResult['status'] !== "error" && $calcResult['STATUS'] !== "API_UNAVAILABLE"){
            $price = $calcResult['body']['price'];
            if(isset($price)){
                if((int)$cart->totalPrice !== (int)$price){
                    $calcResult['data']['state'] = 'processing';
                    $calcResult['PRICE_CHANGED'] = true;
                }
                if($calcResult['BODY'][3]){
                    $calcResult['TERMINAL_RECEIVE_TIME_CHANGED'] = true;
                }
            }
        }

//		echo '<pre>';
//		var_dump($calcResult);
//		echo '</pre>';
//		die();
        return $calcResult;

    }



    /**
     * Получаем sessionId из кэша или авторизуемся
     * */
	public static function getDellinSessionId($arConfig){
		if($arConfig['login'] && $arConfig['password']){
			jimport('joomla.cache.cache');
			$cache_id = 'dellin_shipping';
			$conf =\JFactory::getConfig();
			$options = array(
				'defaultgroup' => 'plg_dellin_shipping',
				'cachebase' => $conf->get('cache_path'),
				'lifetime' => 30*24*60*60,
				'language' => $conf->get('language'),
				'storage' => 'file' );
			$cache = new \JCache($options);
			$cache->setCaching($conf->get('caching'));
			if(!$cache->get($cache_id) || isset($arConfig['reset_session'])){
				$result = self::sendApiRequest('login',$arConfig);

				if(isset($result->sessionID)){
					$sessionId = $result->sessionID;
					$cache->store(serialize($result),$cache_id,'plg_dellin_shipping');
				}


			} else{
				$sessionId = unserialize($cache->get('dellin_shipping'))->sessionID;
			}
			if(isset($sessionId)){
				return $sessionId;
			}else{
				return $result;
			}

		}
		else{
			return false;
		}
	}

	private static function sort_func($a,$b)
	{
		if (strlen($a) == strlen($b))
		{
			//  return 0;
		}

		return (strlen($a) > strlen( $b)) ? -1 : 1;
	}
}
