<?php


trait DellinTrait {

        /**
         * Расчет стоимости доставки.
         * @param array $cart
         * @param array $arConfig
         * @param bool $caching
         * @return array|bool
         */
        public function Calculate($cart, $arConfig, $resetCache = true)
        {


            $data = self::GetDeliveryData($cart, $arConfig);


            if(!isset($data['error'])){
                $validation = DellinApi::IsDataValid($data, $cart, $arConfig);

                if ($validation['status'] == 'ok') {
                    jimport('joomla.cache.cache');
                    $cache_id = 'dellin_shipping_calculate';
                    $conf =\JFactory::getConfig();
                    $options = array(
                        'defaultgroup' => 'plg_dellin_shipping',
                        'cachebase' => $conf->get('cache_path'),
                        'lifetime' => 10*60,
                        'language' => $conf->get('language'),
                        'storage' => 'file' );
                    $cache = new \JCache($options);
                    $cache->setCaching($conf->get('caching'));
                    if($resetCache){
                        $response = DellinApi::sendApiRequest('calculator',$data);
                        if($response == null){
                            echo "<pre>";
                            var_dump($response);
                            echo "</pre>";
                            die();
                        }
                        if(is_array($response) && $response['errors'] == 'Unauthorized'){
                            $result['status'] = "error";
                            $result['body'] = $response->errors->message;
                            return $result;
                        }
                        if (isset($response->errors) && $response->errors !== null) {
                            if (isset($response->errors->message)) {
                                $result['status'] = "api_unavailable";
                                $result['body'] = $response->errors->message;
                                return $result;
                            }else{
                                $result['body'] = "";
                                foreach($response->errors as $index => $error){
                                    if(isset($error->fields)) $errorFields = implode(' , ',$error->fields);
                                    $result['status'] = "error";
                                    $result['body'] .= "[".$error->code."]".$error->detail." Поля: ".$errorFields."| ";
                                }
                            }
                        }else{
                            unset($_SESSION['current_terminals']);
                            $_SESSION['current_terminals'] = $response->data->arrival->terminals;

                            if ($price = self::CalculatePrice($response, $arConfig))
                            {
                                $currentDate = new DateTime(date('Y-m-d'));
                                if($data['delivery']['arrival']['variant'] == 'terminal'){
                                    $arrivalDate = new DateTime ($response->data->orderDates->arrivalToOspReceiver);
                                }else{
                                    $arrivalDate = new DateTime ($response->data->orderDates->derivalFromOspReceiver);
                                }
                                $dateDiff = DellinApi::dateDifference($arrivalDate,$currentDate);
                                if($arConfig['is_goods_unloading'] == ''){
                                    $_SESSION['shipping_days_terminal'] = $dateDiff;
                                } else {
                                    $_SESSION['shipping_days_address'] = $dateDiff;
                                }
                                $result['status'] = 'OK';
                                $result['body']   = array($price, $dateDiff);
                            }
                            if(isset($response->data->availableDeliveryTypes->small) && $arConfig['is_small_goods_price']=="1" ){
                                $result['delivery_type'] = "small";
                            }else{
                                $result['delivery_type'] = "auto";
                            }
                            $cache->store(serialize($result),$cache_id,'plg_dellin_shipping');
                        }
                    }else{
                        $result = unserialize($cache->get($cache_id));
                    }
                    return $result;
                }else{
                    return [
                        'status' => 'error',
                        'errors' => $validation['errors']
                    ];

                }
            }else{
                return [
                    'status'=>'error',
                    'errors'=>$data['errors']
                ];
            }
        }



        protected function getAddressForCDI($cart, $type){

            if($type == 'request'){

                return $cart->person['state'].', '.$cart->person['city'].', '.$cart->person['address'];

            } else {

                $city = isset($cart->ST['city'])?$cart->ST['city']:$cart->BT['city'];
                $state = isset($cart->ST['virtuemart_state_id'])?ShopFunctions::getStateByID($cart->ST['virtuemart_state_id']):
                                                                 ShopFunctions::getStateByID($cart->BT['virtuemart_state_id']);
                $address_1 = isset($cart->ST['address_1'])?$cart->ST['address_1']:$cart->BT['address_1'];
                $address_2 = isset($cart->ST['address_2'])?$cart->ST['address_2']:$cart->BT['address_2'];
                $address = $address_1.' '.$address_2;

                return $state.' '.$city.' '.$address;
            }

        }

        protected static function getCityAndState($cart, $type)
        {

            if($type == 'request'){

                $result = [
                  'state' => $cart->person['state'],
                  'city' => $cart->person['city']
                ];

                return $result;

            } else {
                $city = isset($cart->ST['city'])?$cart->ST['city']:$cart->BT['city'];
                $state = isset($cart->ST['virtuemart_state_id'])?ShopFunctions::getStateByID($cart->ST['virtuemart_state_id']):
                                                           ShopFunctions::getStateByID($cart->BT['virtuemart_state_id']);
                $result = [
                    'city' => $city,
                    'state' => $state
                ];

                return $result;
            }


        }

        protected static function isCashOnDelivery($id){
            if (!class_exists('VirtueMartModelPaymentmethod'))
                require(JPATH_VM_ADMINISTRATOR.DS.'models'.DS.'paymentmethod.php');
            $paymentObj= new VirtueMartModelPaymentmethod;
            $payment = $paymentObj->getPayment($id);
            if($payment->slug == 'cash-on-delivery') return true;
            return false;
        }

        protected static function GetDeliveryData($cart, $arConfig, $type = 'calc'){
            $sessionID = DellinApi::getDellinSessionId($arConfig);
            $deliveryType = 'auto';


            if(isset($cart->ST['city']) || isset($cart->BT['city'])){
                $city = self::getCityAndState($cart, $type)['city'];
                $state = self::getCityAndState($cart, $type)['state'];

                $receiverKladr = DellinApi::GetCityKLADRCode($city, $state);



                if($arConfig['is_small_goods_price'] == '1'){
                    $deliveryType = 'small';
                }
                $opfList = DellinApi::GetOpfList($arConfig['appkey'])['list'];
                if(isset($opfList)){
                    foreach($opfList as $opf){
                        $arOpfList[$opf['uid']] = $opf;
                    }
                }

                //коды ИП для разных стран
                $receiverPPCodes = array(
                    "176" => "0xab91feea04f6d4ad48df42161b6c2e7a"
                );
                $cargoParams = DellinApi::getCargoParams($cart,$arConfig);
                if(!$cargoParams) {
                    $errors['cargo_global_oversize'] = vmText::_ ('VMSHIPMENT_CARGO_GLOBAL_OVERSIZE');
                }
                if($arConfig['delivery_delay']=="" || $arConfig['delivery_delay'] == 0){
                    $deliveryDelay = 1;
                }else{
                    $deliveryDelay = $arConfig['delivery_delay'];
                }
                $produceData = array(
                    "appKey"                => $arConfig['appkey'],
                    "produceDate"           => date("Y-m-d",mktime(0, 0, 0, date("m")  , date("d")+$deliveryDelay, date("Y"))), //ориентировочная минимальная дата (сегодня + дней из настроек)
                    "sizedVolume"           => $cargoParams['totalVolume'],
                    "sizedWeight"           => $cargoParams['totalWeight'],
                    "derivalPoint"          => $arConfig['kladr_code_delivery_from'],
                    "derivalDoor"           => ($arConfig['is_goods_loading'] == "1")?true:false,
                    "deliveryType"          => "1",
                    "derivalPeriodVisit"    => array(
                        "workStart" => $arConfig['work_start'],
                        "workEnd"   => $arConfig['work_end']
                    ),
                    "oversizedWeight" => $cargoParams['oversizedWeight'],
                    "oversizedVolume" => $cargoParams['oversizedVolume'],
                    "length"          => $cargoParams['length'],
                    "width"           => $cargoParams['width'],
                    "height"          => $cargoParams['height']
                );
                $produceDate = DellinApi::GetProduceDate($produceData,$arConfig);
                if(!empty($produceDate)){
                    if(!isset($produceDate->errors)){
                        if($produceDate->nearestDate){//если дата не подошла
                            $nearestDate = $produceDate->nearestDate;
                        }else{
                            $nearestDate = date("Y-m-d",mktime(0, 0, 0, date("m")  , date("d")+$deliveryDelay, date("Y")));
                        }
                    }else{
                        $errors['produce_date_error'] = vmText::_ ('VMSHIPMENT_PRODUCE_DATE_ERROR');
                        $errors['produce_date_errors'] = $produceDate->errors;
                    }
                    $cargoParams['insurance'] = [
                        "statedValue" =>($arConfig['is_insurance_goods_with_declared_price']=='1')?($cart->cartPrices['withTax']):0,
                        "payer"       => self::isCashOnDelivery($cart->virtuemart_paymentmethod_id)?"receiver":"sender",
                        "term"        => false
                    ];
                    $api_data =[
                        'appkey'=>$arConfig['appkey'],
                        'sessionID'=>$sessionID,
                        'delivery'=>[
                            "deliveryType"=>[
                                "type"=>$cargoParams['deliveryType']
                            ],
                            'arrival'=>[],
                            'derival'=>[
                                'produceDate'=>$nearestDate
                            ],
                            'packages'=>[],
                        ],
                        'members'=>[
                            "requester" => [
                                "role"  => "sender",
                            ],
                        ],
                        'cargo'=>$cargoParams,
                        'payment'=>[
                            'paymentCity'=>$arConfig['kladr_code_delivery_from'],
                            'type'=>'noncash'
                        ],
                        'productInfo'=>array(
                            'type'=> 4,
                            'productType'=> 5,
                            'info'=>array(
                                array(
                                    'param'=> 'module version',
                                    'value'=> 'beta'
                                )
                            )
                        )
                    ];


                    if($arConfig['is_goods_loading'] == "1"){
                        $api_data['delivery']['derival']['variant'] = 'address';
                        if(!empty($arConfig['loading_address'])){
                            $api_data['delivery']['derival']['address']['search'] = $arConfig['loading_address'];
                        }else{
                            $api_data['delivery']['derival']['address']['search'] = $arConfig['kladr_code_delivery_from'];
                        }

                       // $api_data['delivery']['derival']['address']['street'] = $arConfig['kladr_code_delivery_from'];
                        $api_data['delivery']['derival']['time'] = array(
                            'worktimeStart' => $arConfig['work_start'],
                            'worktimeEnd'   => $arConfig['work_end'],

                        );
                        if("NULL" !== $arConfig['work_break_start'] && "NULL" !== $arConfig['work_break_end']){
                            $api_data['delivery']['derival']['time']['breakStart'] = $arConfig['work_break_start'];
                            $api_data['delivery']['derival']['time']['breakEnd'] = $arConfig['work_break_end'];

                        }

                    }else{
                        $api_data['delivery']['derival']['variant'] = 'terminal';
                        $api_data['delivery']['derival']['terminalID'] = $arConfig['terminal_id'];
                    }

                    if($arConfig['loading_transport_requirements']!== "NULL") $requirements[] = $arConfig['loading_transport_requirements'];
                    if($arConfig['loading_additional_equipment'] !== "NULL") $requirements[] = $arConfig['loading_additional_equipment'];
                    if(!empty($requirements)) $deliveryData["delivery"]["derival"]['requirements'] = $requirements;

                    if($arConfig['is_goods_unloading'] == "1"){
                        $api_data['delivery']['arrival']['variant'] = 'address';
                        $api_data['delivery']['arrival']['address']['search'] = self::getAddressForCDI($cart, $type);
                        $api_data['delivery']['arrival']['time'] = [
                            'worktimeStart' => $arConfig['work_start'],
                            'worktimeEnd'   => $arConfig['work_end'],
                        ];

                        if("NULL" !== $arConfig['work_break_start'] && "NULL" !== $arConfig['work_break_end']){
                            $api_data['delivery']['arrival']['time']['breakStart'] = $arConfig['work_break_start'];
                            $api_data['delivery']['arrival']['time']['breakEnd'] = $arConfig['work_break_end'];
                        }

                    }else{
                        $api_data['delivery']['arrival']['variant'] = 'terminal';
                        $api_data['delivery']['arrival']['city'] = $receiverKladr;
                    }
                    $arPackages = DellinAPI::GetSelectedPackingTypesId(
                        [
                            'box'=> $arConfig['packing_for_goods_box'],
                            'hard' => $arConfig['packing_for_goods_hard'],
                            'additional' => $arConfig['packing_for_goods_additional'],
                            'bubble' => $arConfig['packing_for_goods_bubble'],
                            'bag' => $arConfig['packing_for_goods_bag'],
                            'pallet' => $arConfig['packing_for_goods_pallet'],
                            'car_glass' =>$arConfig['packing_for_goods_car_glass'],
                            'car_parts' => $arConfig['packing_for_goods_car_parts'],
                            'complex_pallet' => $arConfig['packing_for_goods_complex_pallet'],
                            'complex_hard' => $arConfig['packing_for_goods_complex_hard'],
                        ]
                    );
                    foreach($arPackages as $packageUid){
                        $arPackage = ['uid'=>$packageUid,'payer'=>'sender'];
                        //если мешки или коробки, тоставим количество 1, т.к. поле обязательное, но мы не можем предсказать, сколько их нужно
                        if($packageUid == "0x82750921BC8128924D74F982DD961379"
                            || $packageUid=="0x947845D9BDC69EFA49630D8C080C4FBE"
                            || $packageUid=="0xad97901b0ecef0f211e889fcf4624fed"
                            || $packageUid=="0xad97901b0ecef0f211e889fcf4624fea"
                        ) $arPackage['count']=1;
                        $packages[] = $arPackage;
                    }
                    if(isset($packages)) $api_data["delivery"]["packages"] = $packages;
                    if("NULL" !== $arConfig['counteragent']){
                        $api_data['members']['requester']['uid'] = $arConfig['counteragent'];
                    }
                    return $api_data;
                }else{
                    $errors['api_not_available'] = vmText::_ ('VMSHIPMENT_DELLIN_SHIPPING_API_NOT_AVAILABLE');
                }

            }else{
                $errors['empty_arrival_point'] = vmText::_ ('VMSHIPMENT_EMPTY_ARRIVAL_POINT');
            }
            if(!empty($errors)) return ['error'=>true,'errors'=>$errors];
        }


    protected static function CalculatePrice($response, $arConfig)
    {
        $total_price = 0;
        $is_small_goods_price = ($arConfig['is_small_goods_price'] == '1');
        if ($response->data->price > 0) {
            $total_price = $response->data->price;
        }
        $additional_price = $total_price - $response->data->availableDeliveryTypes->auto;
        if ($is_small_goods_price && isset($response->data->availableDeliveryTypes->small)) {
            $total_price = $response->data->availableDeliveryTypes->small + $additional_price;
        }

        /**/
        return $total_price;
    }


}
